/*
Title:    AndyPi WS2801 DEMO for Arduino Nano
Code by:  Andrew Wilsion http://andypi.co.uk
Lightchaser functions: Andy Mark https://github.com/AndyMark/AndyMark-Arduino-Examples/blob/master/AndyMark_LED_Lightchaser_Example/AndyMark_LED_Lightchaser_Example.ino
Date:     26-11-2014
Version:  1.0

Hardware: 
WS2801 pixel strip: connected to Data Pin and Clock Pin 
Push to make button switch: connected to button pin & GND
Potentiometer controls speed of colour swirl change: connected to analogue pin 

Libraries:
FastlED is the library used to control the LEDS, see here for reference:
https://github.com/FastLED/FastLED/wiki/Pixel-reference



*/

#include "FastLED.h"        // include the FastLED library
#define NUM_LEDS 6          // Define the number of LEDs in the strip
#define MAX_BRIGHTNESS 255  // Define maixmum brightness of the LEDS (0-255)
CRGB leds[NUM_LEDS];        // Define the array of leds
const int buttonPin = 2;    // set constant - pushbutton pin number (Digital pin 2)
int val = 0;                // variable to store the value coming from the potentiometer   

// Define the the digital pin number you attach the WS2801 to on the arduino (13 and 11)
#define DATA_PIN 13
#define CLOCK_PIN 11


void setup() { 
            
      FastLED.addLeds<WS2801, RGB>(leds, NUM_LEDS); // FastLED initialisation using WS2801 chip
      pinMode(buttonPin, INPUT);   // initialize the pushbutton pin as an input:
}


void loop()
{

// uncomment one or more of the functions below to test!
  
// 1. rainbow of colours sweeps along the strip
//rainbow(5);

// 2. Cylon - One LED sweeps back and forth, with the color, delay and number of cycles of your choice
//cylon(CRGB::Red,100, 5);

// 3. Colour Chase - One LED sweeps along the length of the LED strip, with the colour and speed of your choice (once only)
//color_chase(CRGB::Yellow, 150);

// 4. Move an "empty" dot down the strip, with the colour and speed of your choice (once only)
//missing_dot_chase(CRGB::Purple, 200);

// 5. Directly set and LED's 
//leds[2].setHSV(75, 255, 255);  // set LED 2 (third one) to hue=75, Saturation = 255, Brightness = 255
//leds[4].setHSV(250, 255, 255);  // set LED 2 (firth one) to hue=250, Saturation = 255, Brightness = 255
//FastLED.show();

// 6. Colour Fade between colour 1 to colour 2 and back again
// all leds fade through colours specificed (0 red; 32 orange; 64 yellow; 96 green; 128 aqua; 160 blue; 192 purple; 224 pink; 255 red)
colour_fade(160, 255, 50);

}

void pot_sweep() { 

    // read the value from analog pin 2 (the potentiometer) into val
    val = analogRead(2);    
    
    // map the value of the analog input (0-1023) to the HSV colcour spectrum (0-255)
    int myColour = map(val, 0, 1023, 0, 255); 

   for(int dot = 0; dot < NUM_LEDS; dot++) // this loop ensures each "dot" up to the number of LEDs are all changed
    { 
    leds[dot].setHSV(150, 255, 255);  // set hue=myColour, Saturation = 255, Brightness = 255
    }
   FastLED.show();  // update the LEDs

}

//These are the functions we have defined to do chase patterns.  They are actually called inside the  loop() above
//They are meant to demonstrate things such as setting LED colors, controlling brightness
void color_chase(uint32_t color, uint8_t wait)
{
	//clear() turns all LEDs off
	FastLED.clear();
	//The brightness ranges from 0-255
	//Sets brightness for all LEDS at once
	FastLED.setBrightness(MAX_BRIGHTNESS);
	// Move a single led
	for(int led_number = 0; led_number < NUM_LEDS; led_number++)
	{
		// Turn our current led ON, then show the leds
		leds[led_number] = color;

		// Show the leds (only one of which is has a color set, from above
		// Show turns actually turns on the LEDs
		FastLED.show();

		// Wait a little bit
		delay(wait);

		// Turn our current led back to black for the next loop around
		leds[led_number] = CRGB::Black;
	}
	return;
}

//Move an "empty" dot down the strip
void missing_dot_chase(uint32_t color, uint8_t wait)
{
	//Step thru some brightness levels from max to 10.  led_brightness/=2 is a cryptic shorthand way of saying led_brightness = led_brightness/2
	for (int led_brightness = MAX_BRIGHTNESS; led_brightness > 10; led_brightness/=2)
	{
		FastLED.setBrightness(led_brightness);

		// Start by turning all pixels on:
		//for(int led_number = 0; led_number < NUM_LEDS; led_number++) leds[led_number] = color;
		//https://github.com/FastLED/FastLED/wiki/Controlling-leds
		fill_solid(leds, NUM_LEDS, color);

		// Then display one pixel at a time:
		for(int led_number = 0; led_number < NUM_LEDS; led_number++)
		{
			leds[led_number] =  CRGB::Black; // Set new pixel 'off'
			if( led_number > 0 && led_number < NUM_LEDS)
			{
				leds[led_number-1] = color; // Set previous pixel 'on'
			}
			FastLED.show();
			delay(wait);
		}
	}
	return;
}

//Cylon - LED sweeps back and forth, with the color, delay and number of cycles of your choice 
void cylon(CRGB color, uint16_t wait, uint8_t number_of_cycles)
{
	FastLED.setBrightness(255);
	for (uint8_t times = 0; times<=number_of_cycles; times++)
	{
		// Make it look like one LED is moving in one direction
		for(int led_number = 0; led_number < NUM_LEDS; led_number++)
		{
			//Apply the color that was passed into the function
			leds[led_number] = color;
			//Actually turn on the LED we just set
			FastLED.show();
			// Turn it back off
			leds[led_number] = CRGB::Black;
			// Pause before "going" to next LED
			delay(wait);
		}

		// Now "move" the LED the other direction
		for(int led_number = NUM_LEDS-1; led_number >= 0; led_number--)
		{
			//Apply the color that was passed into the function
			leds[led_number] = color;
			//Actually turn on the LED we just set
			FastLED.show();
			// Turn it back off
			leds[led_number] = CRGB::Black;
			// Pause before "going" to next LED
			delay(wait);
		}
	}
	return;
}

void rainbow(uint8_t wait) 
{

	uint16_t hue;
	FastLED.clear();

	for(hue=10; hue<255*3; hue++) 
	{

		fill_rainbow( &(leds[0]), NUM_LEDS /*led count*/, hue /*starting hue*/);		
		FastLED.show();
		delay(wait);
	}
	return;
}


void colour_fade(uint16_t startColour, uint16_t endColour, uint8_t wait)  //start HSV colour, end HSV colour, delay
{  

       FastLED.clear();

       for(int myColour = startColour; myColour < endColour; myColour++) //loop around a colour range from startColour to endColour
       {
         for(int dot = 0; dot < NUM_LEDS; dot++) // this loop ensures each "dot" up to the number of LEDs are all changed
          { 
          leds[dot].setHSV(myColour, 255, 255);  // set hue=myColour, Saturation = 255, Brightness = 255
          }
          FastLED.show();  // update the LEDs
          delay(wait);
       }
       
       for(int myColour = endColour; myColour > startColour; myColour--) //loop around a colour range from the end back to the start
       {
         for(int dot = 0; dot < NUM_LEDS; dot++) // this loop ensures each "dot" up to the number of LEDs are all changed
          { 
          leds[dot].setHSV(myColour, 255, 255);  // set hue=myColour, Saturation = 255, Brightness = 255
          }
          FastLED.show();  // update the LEDs
          delay(wait);
       }

	return;
}
